/*****************************************************************************
 * Copyright (C) The Apache Software Foundation. All rights reserved.        *
 * ------------------------------------------------------------------------- *
 * This software is published under the terms of the Apache Software License *
 * version 1.1, a copy of which has been included with this distribution in  *
 * the LICENSE file.                                                         *
 *****************************************************************************/
 
 /**
  * Program: ViewerFrame.java
  * @author Marlon Pierce
  * Comments:  Various modifications to the original ViewerFrame class from
  * Batik.  
  *
  */

//package org.apache.batik.apps.svgviewer;

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.zip.*;
import javax.swing.*;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;
import java.awt.event.*;

import javax.swing.border.*;
import javax.swing.text.*;

import org.apache.batik.bridge.*;

import org.apache.batik.dom.svg.*;
import org.apache.batik.gvt.event.*;

import org.apache.batik.refimpl.bridge.*;
import org.apache.batik.refimpl.transcoder.*;
import org.apache.batik.refimpl.util.*;
import org.apache.batik.refimpl.gvt.event.*;

import org.apache.batik.util.*;
import org.apache.batik.util.gui.*;

import org.apache.batik.util.gui.resource.*;

import org.xml.sax.*;
import org.w3c.dom.DocumentFragment;

import org.w3c.dom.css.ViewCSS;
import org.w3c.dom.svg.SVGSVGElement;
import org.w3c.dom.svg.SVGAElement;
import org.apache.batik.util.awt.svg.SVGGraphics2D;

//import org.apache.batik.apps.svgviewer.Application;

public class ViewerFrame
    extends    JApplet
    implements org.apache.batik.util.gui.resource.ActionMap,
               UserAgent,
               LanguageChangeHandler,
               UserStyleDialog.ChangeHandler,
               JSVGCanvas.ZoomHandler,
               DocumentListener {

    // The actions names.
    public final static String OPEN_ACTION        = "OpenAction";
    public final static String OPEN_PAGE_ACTION   = "OpenPageAction";
    public final static String NEW_WINDOW_ACTION  = "NewWindowAction";
    public final static String EXPORT_PNG_ACTION  = "ExportPNGAction";
    public final static String EXPORT_JPG_ACTION  = "ExportJPGAction";
    public final static String RELOAD_ACTION      = "ReloadAction";
    public final static String BACK_ACTION        = "BackAction";
    public final static String FORWARD_ACTION     = "ForwardAction";
    public final static String CLOSE_ACTION       = "CloseAction";
    public final static String EXIT_ACTION        = "ExitAction";
    public final static String SOURCE_ACTION      = "SourceAction";
    public final static String DESCRIPTION_ACTION = "DescriptionAction";
    public final static String TREE_ACTION        = "TreeAction";
    public final static String THUMBNAIL_ACTION   = "ThumbnailAction";
    public final static String STOP_ACTION        = "StopAction";
    public final static String FIXED_SIZE_ACTION  = "FixedSizeAction";
    public final static String PROG_PAINT_ACTION  = "ProgressivePaintAction";
    public final static String LANGUAGE_ACTION    = "LanguageAction";
    public final static String USER_STYLE_ACTION  = "UserStyleAction";
    public final static String MONITOR_ACTION     = "MonitorAction";
    public final static String ABOUT_ACTION       = "AboutAction";

    /**
     * The following data are accessible by all of the methods of
     * the ViewerFrame class.  The names are self explanatory.  These
     * variables are typically initialized in the constructor or in
     * init().
     */
    protected final static Cursor DEFAULT_CURSOR =
        new Cursor(Cursor.DEFAULT_CURSOR);

    protected final static Cursor WAIT_CURSOR =
        new Cursor(Cursor.WAIT_CURSOR);

    public final static String RESOURCES =
	//        "org.apache.batik.apps.svgviewer.resources.GUI";
        "resources.GUI";

    protected static JFrame memoryMonitor;
    protected static JFrame aboutFrame;
    protected static ResourceBundle bundle;
    protected static ResourceManager resources;
    static {
        bundle    = ResourceBundle.getBundle(RESOURCES, Locale.getDefault());
        resources = new ResourceManager(bundle);
    }
    
    protected static char [] buffer = new char[4096];
    protected Application application;
    protected Main main;
    protected LocationBar locationBar;
    protected JButton textButton;
    protected String uri;
    protected String currentPath = ".";
    protected String currentExportPath = ".";
    protected URIChooser uriChooser;
    protected JPanel panel;
    protected StatusBar statusBar;
    protected JSVGCanvas canvas,textcanvas;
    protected Thread thread;
    protected DOMViewer domViewer = new DOMViewer();
    protected LanguageDialog languageDialog;
    protected UserStyleDialog userStyleDialog;
    protected JFrame thumbnailFrame;
    
    protected ReloadAction reloadAction = new ReloadAction(this);
    protected BackAction backAction = new BackAction(this);
    protected ForwardAction forwardAction = new ForwardAction(this);
    protected StopAction stopAction = new StopAction(this);
    protected IntegralAction intAction=new IntegralAction();
    protected LoadDataAction loadAction=new LoadDataAction();

    protected String description = "";
    protected boolean fixedSize;
    protected boolean progressivePaintEnabled;
    
    private ConcreteEventDispatcher eventDispatcher =
        new ConcreteEventDispatcher();

    protected String userLanguages = "en";
    
    protected String userStyleSheetURI;
    protected java.util.List loadedDocuments = new java.util.ArrayList();
    protected int loadedDocument = -1;
    protected int loadedDocumentsCount;
    private SVGDocumentFactory df;
    public SVGOMDocument svgomdoc=new SVGOMDocument();
    protected DefaultSVGContext dc;

    public JMenuBar jmb;
    public JToolBar tb,mb;
    public MyJEditorPane mep;
    public JPanel container,northpane;
    public MyJTextArea jta;
    
    protected Font font;
    int increment;
    
    /**
     * Creates a new ViewerFrame object.
     * @param a The current application.
     */
    public ViewerFrame(Main main) {
	//super(new BorderLayout());
        this.main=main;
        font=new Font("TimesRoman",Font.BOLD,32);

        listeners.put(OPEN_ACTION,        new OpenAction());
        listeners.put(OPEN_PAGE_ACTION,   new OpenPageAction());
        listeners.put(NEW_WINDOW_ACTION,  new NewWindowAction(this));
        listeners.put(EXPORT_PNG_ACTION,  new ExportPNGAction(this));
        listeners.put(EXPORT_JPG_ACTION,  new ExportJPGAction(this));
        listeners.put(RELOAD_ACTION,      reloadAction);
        listeners.put(BACK_ACTION,        backAction);
        listeners.put(FORWARD_ACTION,     forwardAction);
        listeners.put(CLOSE_ACTION,       main.createCloseAction(this));
        listeners.put(EXIT_ACTION,        main.createExitAction());
        listeners.put(SOURCE_ACTION,      new SourceAction(this));
        listeners.put(DESCRIPTION_ACTION, new DescriptionAction(this));
        listeners.put(TREE_ACTION,        new TreeAction(this));
        listeners.put(THUMBNAIL_ACTION,   new ThumbnailAction(this));
        listeners.put(STOP_ACTION,        stopAction);
        listeners.put(PROG_PAINT_ACTION,  new ProgressivePaintAction());
        listeners.put(FIXED_SIZE_ACTION,  new FixedSizeAction(this));
        listeners.put(LANGUAGE_ACTION,    new LanguageAction(this));
        listeners.put(USER_STYLE_ACTION,  new UserStyleAction(this));
        listeners.put(MONITOR_ACTION,     new MonitorAction(this));
        listeners.put(ABOUT_ACTION,       new AboutAction(this));

	listeners.put("IntegralAction", intAction);
	listeners.put("SqrtAction",new SqrtAction());
	listeners.put("LoadDataAction",loadAction);
	listeners.put("SaveSVGAction",new SaveSVGAction());
    }
    
    /**
     * Put all of the initialization code here.
     */
    public void init(){       
	//Get the bounds of the main frame and establish the
	//size of the menu and tool bars.
	int boundX=main.getXDim();
	int boundY=main.getYDim();
	
	//Initialize the document factory.
        df = new SVGDocumentFactory(main.getXMLParserClassName());

	SVGGraphics2D svgbase=new SVGGraphics2D(new SVGOMDocument());
	try{
	    CharArrayWriter out=new CharArrayWriter();
	    BufferedWriter bufOut=new BufferedWriter(out);
	    svgbase.stream(bufOut,true);
	    char[] charArray=out.toCharArray();
	    CharArrayReader charReader=new CharArrayReader(charArray);
	    BufferedReader bufRead=new BufferedReader(charReader);
  	    svgomdoc=df.createDocument("file:/tmp",new InputSource(bufRead));
	    out.flush();
	    out.close();
	    bufOut.flush();
	    bufOut.close();
	}
	catch(Exception ex) {
	    ex.printStackTrace();
	}

	//Create the container pane that holds all of these components.
	container=new JPanel(new BorderLayout());
	northpane=new JPanel(new BorderLayout());
	 	
	//Give reference of container JFrame to local instance.
	uriChooser = new URIChooser(main, new URIChooserOKAction());
        uriChooser.setFileFilter(new SVGFileFilter());
	
        //Create the SVG canvas and other containers.  Note this MUST
	//come first in order for the action objects above to work.
        canvas = new JSVGCanvas(this);

        //Create the menu
	MenuFactory mf = new MenuFactory(bundle, this);
	jmb=mf.createJMenuBar("MenuBar");

        //Create the toolbar
	ToolBarFactory tbf = new ToolBarFactory(bundle, this);
	tb = tbf.createJToolBar("ToolBar");
	tb.setFloatable(true);
	
	//Add these two components to the north panel.
	northpane.add("North",jmb);
	northpane.add("South",tb);

        // Create the status bar
	statusBar=new StatusBar();
	
        //Create the language dialog
        languageDialog = new LanguageDialog(main);
        languageDialog.setLanguageChangeHandler(this);
	
        // Create the user style dialog
        userStyleDialog = new UserStyleDialog(main);
        userStyleDialog.setChangeHandler(this);
	
	//Add the north pane, canvas, and status bar.
	container.add("North",northpane);
	container.add("Center",new JScrollPane(canvas));
	container.add("South",statusBar);

	// Create the editor pane.  I will move this somewhere more 
	// appropriate later, I think.
	// Create a math toolbar
	JPanel editpane=new JPanel(new BorderLayout());
	ToolBarFactory mathbar=new ToolBarFactory(bundle,this);
	mb=mathbar.createJToolBar("MathBar");
	mb.setFloatable(true);

//  	mep=new MyJEditorPane(main);
//  	mep.setContentType("text/html");
//  	mep.setEditable(true);
	
	jta=new MyJTextArea(main);

	editpane.add("North",mb);
	//	editpane.add("Center",mep);
	editpane.add("Center",new JScrollPane(jta));

	Container mainContain=main.getContentPane();
	mainContain.setLayout(new GridLayout(2,1));
  	mainContain.add(container);
	mainContain.add(editpane);

	//Add event listeners to the canvas.
        canvas.setZoomHandler(this);
        canvas.addMouseMotionListener(new MouseMotionAdapter() {
                public void mouseMoved(MouseEvent e) {
                    statusBar.setXPosition(e.getX());
                    statusBar.setYPosition(e.getY());
                }
            });
        canvas.addMouseListener(new MouseAdapter() {
                public void mouseExited(MouseEvent e) {
                    Dimension dim = canvas.getSize();
                    statusBar.setWidth(dim.width);
                    statusBar.setHeight(dim.height);
                }
            });
        canvas.addComponentListener(new ComponentAdapter() {
                public void componentResized(ComponentEvent e) {
                    Dimension dim = canvas.getSize();
                    statusBar.setWidth(dim.width);
                    statusBar.setHeight(dim.height);
                }
            });
    }
    
    /**
     * Tells the viewer whether or not it must be set to the size
     * of the loaded documents.
     */
    public void setFixedSize(boolean b) {
        fixedSize = b;
        // !!! TODO
    }

    /**
     * Returns the fixedSize field value.
     */
    public boolean isFixedSize() {
        return fixedSize;
    }

    /**
     * Returns the current loading thread if one.
     */
    public Thread getLoadingThread() {
        return thread;
    }

    /**
     * Called when the language settings change.
     */
    public void languageChanged(String lang) {
        userLanguages = lang;
    }

    /**
     * Called when the user stylesheet has changed.
     */
    public void userStyleSheetURIChanged(String s) {
        userStyleSheetURI = s;
    }

    public void zoomChanged(float f) {
        statusBar.setZoom(f);
    }

    // UserAgent ///////////////////////////////////////////////////

    /**
     * Returns the default size of the viewport of this user agent.
     */
    public Dimension2D getViewportSize() {
        return canvas.getSize();
    }

    /**
     * Returns the <code>EventDispatcher</code> used by the
     * <code>UserAgent</code> to dispatch events on GVT.
     */
    public EventDispatcher getEventDispatcher() {
        return eventDispatcher;
    }

    /**
     * Displays an error message in the User Agent interface.
     */
    public void displayError(final String msg) {
        Runnable r = new Runnable() {
            public void run() {
                JOptionPane pane =
                    new JOptionPane(msg, JOptionPane.ERROR_MESSAGE);
                JDialog dialog = pane.createDialog(ViewerFrame.this, "ERROR");
                dialog.show();
            }
        };
        if (!EventQueue.isDispatchThread()) {
            EventQueue.invokeLater(r);
        } else {
            r.run();
        }
    }

    /**
     * Displays an error resulting from the specified Exception.
     */
    public void displayError(final Exception ex) {
        displayError(ex.getMessage());
    }

    /**
     * Displays a message in the User Agent interface.
     */
    public void displayMessage(String message) {
        statusBar.setMainMessage(message);
    }

    /**
     * Returns the pixel to mm factor.
     */
    public float getPixelToMM() {
        return 0.33f;
    }

    /**
     * Returns the language settings.
     */
    public String getLanguages() {
        return userLanguages;
    }

    /**
     * Returns the user stylesheet uri.
     * @return null if no user style sheet was specified.
     */
    public String getUserStyleSheetURI() {
        return userStyleSheetURI;
    }

    /**
     * Opens a link.
     * @param elt The activated link element.
     */
    public void openLink(SVGAElement elt) {
        main.openLink(this, elt);
    }

    /**
     * Informs the user agent to change the cursor.
     * @param cursor the new cursor
     */
    public void setSVGCursor(Cursor cursor) {
        // XXX: immediate set,  but may not update "requested" value
        canvas.setCursor(cursor);
    }

    /**
     * Runs the given thread.
     */
    public synchronized void runThread(Thread t) {
        stopAction.update(true);
        reloadAction.update(false);
        thread = t;
        thread.start();
    }

    /**
     * Returns the class name of the XML parser.
     */
    public String getXMLParserClassName() {
        return main.getXMLParserClassName();
    }

    /**
     * Returns the <code>AffineTransform</code> currently
     * applied to the drawing by the UserAgent.
     */
    public AffineTransform getTransform() {
        return canvas.getTransform();
    }

    /**
     * Returns the location on the screen of the
     * client area in the UserAgent.
     */
    public Point getClientAreaLocationOnScreen() {
        return canvas.getLocationOnScreen();
    }

    /**
     * Loads the given document.
     * @param s The document name.
     */
    public void loadDocument(String s) {
        uri = s;
        File f = new File(uri);
        if (f.exists()) {
            if (f.isDirectory()) {
                uri = null;
            } else {
                uri = "file:" + uri;
            }
        }
        if (uri != null) {
            // interrupt any document load already underway
            if ((thread != null) && thread.isAlive()) {
                thread.interrupt();
            }
            loadedDocument++;
            if (loadedDocument == loadedDocuments.size()) {
                loadedDocuments.add(uri);
            } else {
                Object o = loadedDocuments.get(loadedDocument);
                if (!o.equals(uri)) {
                    loadedDocumentsCount = loadedDocument + 1;
                }
                loadedDocuments.set(loadedDocument, uri);
            }
            if (loadedDocumentsCount == loadedDocument) {
                loadedDocumentsCount++;
            }
            backAction.update();
            forwardAction.update();

//              locationBar.setText(uri);
            Thread t = DocumentLoadRunnable.createLoaderThread(uri,this,df);
            runThread(t);
        }
    }

    /**
     * Append the fragment to the main svg document.
     */
    public void appendFragToDoc(SVGOMDocument textdoc) {

	//    	loadInternalDocument(svgomdoc);

  	svgomdoc=textdoc;

	setCursor(WAIT_CURSOR);
	setSVGCursor(WAIT_CURSOR);
	canvas.setSVGDocument(null);
	reloadAction.update(false);
	stopAction.update(true);
	statusBar.setMainMessage(resources.getString("Document.loading"));
	
	//New doc has been loaded, prepare for new view
	//DefaultSVGContext dc = new DefaultSVGContext();
	dc=new DefaultSVGContext();
//  	dc.setUserAgent(ViewerFrame.this);
  	dc.setUserAgent(this);
	dc.setUserStyleSheetURI(userStyleSheetURI);

    	svgomdoc.setSVGContext(dc);
    	domViewer.setDocument(svgomdoc,
    			      (ViewCSS)svgomdoc.getDocumentElement());
    	statusBar.setMainMessage(resources.getString("Document.creating"));
    	canvas.setSVGDocument(svgomdoc);
	
	setCursor(DEFAULT_CURSOR);
	stopAction.update(false);
	reloadAction.update(true);
	statusBar.setMainMessage("");
	statusBar.setMessage(resources.getString("Document.done"));
    }

    /**
     * This method loads an SVGOMDocument that was created internally
     * by another component to the specified canvas.
     */
    public void loadInternalDocument(SVGOMDocument doc) {

	setCursor(WAIT_CURSOR);
	setSVGCursor(WAIT_CURSOR);
	canvas.setSVGDocument(null);
	reloadAction.update(false);
	stopAction.update(true);
	statusBar.setMainMessage(resources.getString("Document.loading"));
	
	//New doc has been loaded, prepare for new view
	//DefaultSVGContext dc = new DefaultSVGContext();
	dc = new DefaultSVGContext();
  	svgomdoc = doc;
	
	//	main.setSVGOMDocument(svgomdoc);
	dc.setUserAgent(ViewerFrame.this);
	dc.setUserStyleSheetURI(userStyleSheetURI);
	svgomdoc.setSVGContext(dc);
	domViewer.setDocument(svgomdoc,
			      (ViewCSS)svgomdoc.getDocumentElement());
	statusBar.setMainMessage(resources.getString("Document.creating"));
	
	canvas.setSVGDocument(svgomdoc);
	setCursor(DEFAULT_CURSOR);
	stopAction.update(false);
	reloadAction.update(true);
	statusBar.setMainMessage("");
	statusBar.setMessage(resources.getString("Document.done"));

    }
    
    // ActionMap /////////////////////////////////////////////////////

    /**
     * The map that contains the action listeners
     */
    protected Map listeners = new HashMap();

    /**
     * Returns the action associated with the given string
     * or null on error
     * @param key the key mapped with the action to get
     * @throws MissingListenerException if the action is not found
     */
    public Action getAction(String key) throws MissingListenerException {
        Action result = (Action)listeners.get(key);
        if (result == null) {
            result = canvas.getAction(key);
        }
        return result;
    }

    /**
     * Closes the frame.
     */
    public void close() {
        if (main.closeFrame(this)) {
            setVisible(false);
        }
    }


    // Actions //////////////////////////////////////////////////////////

    /**
     * Take action on receipt of a document event.
     */
    public void processDocumentEvent(DocumentEvent e) {

        // perhaps keying on public field is faster than instanceof ?

        if (e.classid == DocumentEvent.LOADING) {
            processDocumentLoadingEvent((DocumentLoadingEvent)e);
        } else if (e.classid == DocumentEvent.PROPERTY) {
            processDocumentPropertyEvent((DocumentPropertyEvent)e);
        }
    }

    /**
     * Take action on receipt of a document loading event.
     */
    public void processDocumentLoadingEvent(DocumentLoadingEvent e) {
        switch (e.type) {
        case (DocumentLoadingEvent.START_LOADING):
            setCursor(WAIT_CURSOR);
            setSVGCursor(WAIT_CURSOR);
            canvas.setSVGDocument(null);
            reloadAction.update(false);
            stopAction.update(true);
            statusBar.setMainMessage(resources.getString("Document.loading"));
            break;
        case (DocumentLoadingEvent.LOADED):
            // New doc has been loaded, prepare for new view
            DefaultSVGContext dc = new DefaultSVGContext();
            svgomdoc = (SVGOMDocument) e.getValue();
	    //	    main.setSVGOMDocument(svgomdoc);
            dc.setUserAgent(ViewerFrame.this);
            dc.setUserStyleSheetURI(userStyleSheetURI);
            svgomdoc.setSVGContext(dc);
            domViewer.setDocument(svgomdoc,(ViewCSS)svgomdoc.getDocumentElement());
            statusBar.setMainMessage(resources.getString("Document.creating"));
            break;
        case (DocumentLoadingEvent.DONE):
            svgomdoc = (SVGOMDocument) e.getValue();
	    main.setSVGOMDocument(svgomdoc);
            canvas.setSVGDocument(svgomdoc);
            setCursor(DEFAULT_CURSOR);
            stopAction.update(false);
            reloadAction.update(true);
            statusBar.setMainMessage("");
            statusBar.setMessage(resources.getString("Document.done"));
            break;
        case (DocumentLoadingEvent.LOAD_CANCELLED):
            setCursor(DEFAULT_CURSOR);
            stopAction.update(false);
            reloadAction.update(true); // in case we want to try again
            statusBar.setMainMessage("");
            statusBar.setMessage(resources.getString("Document.cancelled"));
            break;
        case (DocumentLoadingEvent.LOAD_FAILED):
            setCursor(DEFAULT_CURSOR);
            stopAction.update(false);
            reloadAction.update(true); // in case we want to try again
            statusBar.setMainMessage("");
            statusBar.setMessage(resources.getString("Document.failed"));
            displayError(e.getException());
        }
    }

    public void writeOutDoc(SVGOMDocument testdoc) {

	try {
  	    Writer stdout=new OutputStreamWriter(System.out,"UTF-8");
  	    System.out.println("Updated document");
	    XmlWriter.writeXml(testdoc,stdout);
	    stdout.flush();
	}
	catch(Exception ex) {ex.printStackTrace();}

    }

    /**
     * Take action on receipt of a document property change.
     */
    //Comment for now while converting ViewerFrame to JApplet.
    public void processDocumentPropertyEvent(DocumentPropertyEvent e) {
//  	switch (e.type) {
//  	case (DocumentPropertyEvent.TITLE) :
//  	    String title = (String) e.getValue();
//            if (title.equals("")) {
//                setTitle(resources.getString("Frame.title") + ": " +
//                         resources.getString("Frame.no_title"));
//            } else {
//                setTitle(resources.getString("Frame.title") + ": " + title);
//            }
//            break;
//  	case (DocumentPropertyEvent.SIZE) :
//  	    Dimension size = (Dimension) e.getValue();
//  	    if (!canvas.getSize().equals(size)) {
//  		canvas.setPreferredSize(size);
//  		panel.invalidate();
//  		if (!fixedSize) {
//  		    pack();
//  		}
//  	    }
//  	    break;
//  	case (DocumentPropertyEvent.DESCRIPTION) :
//  	    String desc = (String) e.getValue();
//  	    if (desc.equals("")) {
//  		desc = resources.getString("Description.no_description");
//  	    }
//  	    description = desc;
//  	    break;
//  	}
    }
    

    /**
     * Define below the accessor methods.
     */   
    public String getDescription() { return description; }
    public String getUri() { return uri; }
    public ResourceManager getResources() { return resources; }
    
    public int getLoadedDocument() { return loadedDocument; }
    public java.util.List getLoadedDocumentList() { return loadedDocuments; }
    public int getLoadedDocumentsCount() { return loadedDocumentsCount; }

    public JSVGCanvas getJSVGCanvas() { return canvas; }
    public String getCurrentExportPath() { return currentExportPath; }
    public StatusBar getStatusBar() { return statusBar; }

    public boolean getFixedSize() { return fixedSize; }
    
    public LanguageDialog getLanguageDialog() { return languageDialog; }
    public String getUserLanguages() { return userLanguages; }

    public LocationBar getLocationBar() { return locationBar; }

    public MemoryMonitor getMemoryMonitor() { 
	return (MemoryMonitor)memoryMonitor; 
    }

    public Main getMain() { return main; }
    public Application getApplication() {return main; }

    public URIChooser getURIChooser() { return uriChooser; }
    
    public boolean getProgressivePaintEnabled() {
	return progressivePaintEnabled; 
    }

    public SVGDocumentFactory getDocFactory() { return df; }
    
    public Thread getThread() { return thread; }

    public JFrame getThumbnailFrame() { return thumbnailFrame; }

    public DOMViewer getDOMViewer() { return domViewer; }
    
    public UserStyleDialog getUserStyleDialog() { return userStyleDialog; }

    public char[] getBuffer() { return buffer; }

    public String getCurrentPath() { return currentPath; }
    
    public ResourceBundle getBundle() { return bundle; }
	
    public JPanel getContainerFrame() { return container; }
    
    public JPanel getpanel() { return panel; }
    
    public JButton getTextButton() { return textButton; }

    public MyJEditorPane getMyJEditorPane() { return mep; }

    public SVGOMDocument getsvgomdoc() { return svgomdoc; }
    public void setSVGDocument(SVGOMDocument doc) { svgomdoc=doc; }



    /**============================================================
     * The following inner classes define actions for the buttons.
     *============================================================*/
    
    /**
     * Define the open page action.
     */
    public class OpenPageAction extends AbstractAction {
	
	public OpenPageAction() {}
	
	public void actionPerformed(ActionEvent e) {
	    uriChooser.pack();
	    
	    Rectangle fr = getBounds();
	    Dimension ud = uriChooser.getSize();
	    uriChooser.setLocation(fr.x + (fr.width  - ud.width) / 2,
				   fr.y + (fr.height - ud.height) / 2);
	    
	    uriChooser.show();
	}
    }
    
    /**
     * The URIChooserOKAction definition.
     */
    public class URIChooserOKAction extends AbstractAction {
	public URIChooserOKAction() {}
	public void actionPerformed(ActionEvent e) {
	    loadDocument(uriChooser.getText());
	}
    }

    /**
     * The action to place the integral sign.
     */
    public class IntegralAction extends AbstractAction {
	public IntegralAction() {}
	public void actionPerformed(ActionEvent e) {
	    jta.append("<int>");

	    //Don't do anything yet.
	    //	    mep.insertComponent(new JButton("test button"));
	    //mep.setText("<int>");
	}
    }

    /**
     * The action to place the square root sign.
     */
    public class SqrtAction extends AbstractAction {
	public SqrtAction() {}
	public void actionPerformed(ActionEvent e) {
	    jta.append("<sqr>");
	}
    }

    public class ProgressivePaintAction extends AbstractAction {
	public ProgressivePaintAction() {}

	public void actionPerformed(ActionEvent e) {
	    progressivePaintEnabled=
		((JCheckBoxMenuItem)e.getSource()).isSelected();
	    canvas.setProgressiveRenderingEnabled(progressivePaintEnabled);
	}
    }
    
    /**
     * Opens an existing SVG document from disk.
     */
    public class OpenAction extends AbstractAction {
	public OpenAction() {}

	public void actionPerformed(ActionEvent e) {
	    JFileChooser fileChooser =
		new JFileChooser(currentPath);
	    fileChooser.setFileHidingEnabled(false);
	    fileChooser.setFileSelectionMode
		(JFileChooser.FILES_AND_DIRECTORIES);
	    fileChooser.setFileFilter(new SVGFileFilter());
	    
	    int choice = fileChooser.showOpenDialog(ViewerFrame.this);
	    if (choice == JFileChooser.APPROVE_OPTION) {
		File f = fileChooser.getSelectedFile();
		try {
		    loadDocument(currentPath = f.getCanonicalPath());
		} 
		catch (IOException ex) {
		}
	    }
	}
    }

    /**
     * Loads a data file and displays it.
     */
    public class LoadDataAction extends AbstractAction {
	public LoadDataAction() {}

	public void actionPerformed(ActionEvent e) {
	    GraphicsManager graphman;
	    String[] placebo=new String[1];
	    JFileChooser fileChooser =
		new JFileChooser(currentPath);
	    fileChooser.setFileHidingEnabled(false);
	    fileChooser.setFileSelectionMode
		(JFileChooser.FILES_AND_DIRECTORIES);
	    fileChooser.setFileFilter(new DataFileFilter());
	    
	    int choice = fileChooser.showOpenDialog(ViewerFrame.this);
	    if (choice == JFileChooser.APPROVE_OPTION) {
		File f = fileChooser.getSelectedFile();
		try {
		    System.out.println("Load the data file and grok");
		    graphman=main.getGraphicsManager();
		    graphman.test();
  		    graphman.initChildren();
		    placebo[0]=f.getCanonicalPath();
		    graphman.loadDataFiles(placebo);
 		    graphman.drawChildren(main.getSVGGenerator());
		} 
		catch (Exception ex) {
		}
	    }
	}
    }

    /**
     * Saves the current SVG image to a file.
     */
    public class SaveSVGAction extends AbstractAction {
	public SaveSVGAction() {}

	public void actionPerformed(ActionEvent e) {
	    JFileChooser fileChooser =
		new JFileChooser(currentPath);
	    fileChooser.setFileHidingEnabled(false);
	    fileChooser.setFileSelectionMode
		(JFileChooser.FILES_AND_DIRECTORIES);
	    fileChooser.setFileFilter(new SVGFileFilter());
	    
	    int choice = fileChooser.showSaveDialog(ViewerFrame.this);
	    if (choice == JFileChooser.APPROVE_OPTION) {
		File f = fileChooser.getSelectedFile();

  		try {
		    Writer writeout=new PrintWriter(new FileWriter(f));
  		    XmlWriter.writeXml(svgomdoc,writeout);
		    writeout.flush();
  		} 
  		catch (Exception ex) {
  		}
	    }
	}
    }

    /**
     * Exports the image as JPG.
     */
    public class ExportJPGAction extends AbstractAction {
	public ExportJPGAction(ViewerFrame vf){
	}
	public void actionPerformed(ActionEvent e) {
	    JFileChooser fileChooser =
		new JFileChooser(currentExportPath);
            fileChooser.setFileHidingEnabled(false);
            fileChooser.setFileSelectionMode
                (JFileChooser.FILES_AND_DIRECTORIES);
	    
            int choice = fileChooser.showSaveDialog(ViewerFrame.this);
            if (choice == JFileChooser.APPROVE_OPTION) {
                final File f = fileChooser.getSelectedFile();
                BufferedImage buffer = canvas.getBuffer();
                // create a BufferedImage of the appropriate type
                int w = buffer.getWidth();
                int h = buffer.getHeight();
                final ImageTranscoder trans = new JpegTranscoder();
                final BufferedImage img = trans.createImage(w, h);
                // paint the buffer to the image
                Graphics2D g2d = img.createGraphics();
                g2d.drawImage(buffer, null, 0, 0);
                new Thread() {
			public void run() {
			    try {
				currentExportPath = f.getCanonicalPath();
				OutputStream ostream =
				    new BufferedOutputStream(new FileOutputStream(f));
				trans.writeImage(img, ostream);
				ostream.flush();
				ostream.close();
				statusBar.setMessage(
						     resources.getString("Document.export"));
			    } catch (IOException ex) { }
			}
		    }.start();
            }
	}
    }
    
}



